@@ -4,7 +4,7 @@
   FILE: icalvalue.c
   CREATOR: eric 02 May 1999
   
-  $Id: icalvalue.c,v 1.21 2003/10/21 18:28:27 ettore Exp $
+  $Id: icalvalue.c,v 1.24 2006/12/03 17:18:02 kharish Exp $
 
 
  (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
@@ -1318,5 +1318,44 @@ int icalvalue_encode_ical_string(const char *szText, char *szEncText, int nMaxBu
     return 1;
 }
 
+int icalvalue_decode_ical_string(const char *szText, char *szDecText, int nMaxBufferLen)
+{
+    char *str, *str_p;
+    const char *p;
+    icalvalue *value = 0;
+    size_t buf_sz;
+
+    if ((szText == 0) || (szDecText == 0))
+        return 0;
+	
+    buf_sz = strlen(szText);
+    str_p = str = (char*)icalmemory_new_buffer(buf_sz + 1);
+
+    if (str_p == 0){
+        return 0;
+    }
+
+    for (p=szText; *p!=0; p++) {
+	if (*p == '\\') {
+	    icalmemory_append_char (&str,&str_p,&buf_sz,*(p+1));
+	    p++;
+	}	    
+	else
+            icalmemory_append_char (&str,&str_p,&buf_sz,*p);
+    }	    
+    
+    icalmemory_append_char(&str,&str_p,&buf_sz,'\0');
+
+    if ((int)strlen(str) > nMaxBufferLen) {
+            return 0;
+    }
+
+    strcpy(szDecText, str);
+
+    icalmemory_free_buffer(str);	
+    return 1;
+}
+
+
 /* The remaining interfaces are 'new', 'set' and 'get' for each of the value
    types */
